<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Questions list with pattern page
 * @package     quizaccess_quilgo
 * @copyright   2023 Native Platform Ltd <hello@quilgo.com>
 * @license     http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

require_once(__DIR__ . '/../../../../config.php');
require_once($CFG->dirroot . '/mod/quiz/locallib.php');
require_once($CFG->dirroot. '/mod/quiz/accessrule/quilgo/lib.php');

$cmid = required_param('cmid', PARAM_INT);
$attemptid = required_param('attemptid', PARAM_INT);
$patternsQuery = required_param('patterns', PARAM_RAW);

$context = context_module::instance($cmid, MUST_EXIST);
$attemptobj = quiz_create_attempt_handling_errors($attemptid, $cmid);
$attemptobj->preload_all_attempt_step_users();

require_login($attemptobj->get_course(), false, $attemptobj->get_cm());

$patternPerQuestion = [];
if (strlen($patternsQuery) > 1) {
    $patterns = explode("|", $patternsQuery);
    foreach($patterns as $pattern) {
        $p = new stdClass;
        $explodedPattern = explode("#", $pattern);
        $p->questionId = $explodedPattern[0];
        $p->type = $explodedPattern[1];
        $patternPerQuestion[] = $p;
    }
}
            
$renderer = $PAGE->get_renderer('mod_quiz');
$slots = $attemptobj->get_slots();

function build_question_pattern_steps($steps) {
    $stepIcon = html_writer::tag('i', '', ['class' => 'fa fa-arrow-right mr-1']);
    
    $stepContents = "";
    foreach ($steps as $key => $step) {
        $badge = $key > 1 ? 'badge-secondary' : 'badge-danger';
        $stepContents .= html_writer::tag(
            'span',
            $step,
            ['class' => "badge $badge mr-1"],
        );

        if ($key < count($steps) - 1) {
            $stepContents .= $stepIcon;
        }
    }

    return $stepContents;
}

$questionContents = "";
foreach ($slots as $slot) {
    $questionAttempt = $attemptobj->get_question_attempt($slot);
    $outeruniqueid = $questionAttempt->get_outer_question_div_unique_id();
    $questionContents .= $attemptobj->render_question($slot, false, $renderer);

    $patternQuestionContents = "";
    $patternPerQuestionIndex = array_search($outeruniqueid, array_column($patternPerQuestion, 'questionId'));
    if ($patternPerQuestionIndex !== false) {
        $pattern = $patternPerQuestion[$patternPerQuestionIndex];
        $patternQuestionContents .= html_writer::tag(
            'div',
            get_string('report_patterns_detected', 'quizaccess_quilgo'),
            ['class' => 'text-danger font-weight-bold']
        );

        $stepsContents = "";
        $steps = [];
        if ($pattern->type == QUIZACCESS_QUILGO_PATTERN_TYPE_COPY_PASTE) {
            $steps = [
                get_string('report_patterns_step_copy_question', 'quizaccess_quilgo'),
                get_string('report_patterns_step_leave_test', 'quizaccess_quilgo'),
                get_string('report_patterns_step_return', 'quizaccess_quilgo'),
                get_string('report_patterns_step_paste_answer', 'quizaccess_quilgo')
            ];
        } else if ($pattern->type == QUIZACCESS_QUILGO_PATTERN_TYPE_COPY_CHANGE) {
            $steps = [
                get_string('report_patterns_step_copy_question', 'quizaccess_quilgo'),
                get_string('report_patterns_step_leave_test', 'quizaccess_quilgo'),
                get_string('report_patterns_step_return', 'quizaccess_quilgo'),
                get_string('report_patterns_step_change_answer', 'quizaccess_quilgo')
            ];
        }

        $stepsContents .= build_question_pattern_steps($steps);
        $stepsContents = html_writer::tag(
            'div',
            $stepsContents,
            ['class' => 'd-flex align-items-center flex-wrap']
        );

        $patternQuestionContents = html_writer::tag(
            'div',
            $patternQuestionContents . $stepsContents,
            ['class' => 'content']
        );

        $patternQuestionContents = html_writer::tag(
            'div',
            $patternQuestionContents,
            ['class' => 'que mb-5 pb-5']
        );
    }

    $questionContents .= $patternQuestionContents;
}

$questionUrl = quizaccess_quilgo_get_questions_url($cmid, $attemptid, $patternsQuery);
$PAGE->set_url($questionUrl);
$PAGE->set_pagelayout('embedded');
$PAGE->requires->css('/mod/quiz/accessrule/quilgo/styles.css');

echo $OUTPUT->header();
echo $questionContents;
echo $OUTPUT->footer();
